@@ -3,7 +3,7 @@
 *                C o m p o s i t e   W i n d o w   O b j e c t                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXComposite.cpp,v 1.49 2005/01/16 16:06:06 fox Exp $                     *
+* $Id: FXComposite.cpp,v 1.54.2.1 2006/05/10 13:18:13 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -92,8 +92,8 @@ FXComposite::FXComposite(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXin
 
 
 // Is widget a composite
-FXbool FXComposite::isComposite() const {
-  return 1;
+bool FXComposite::isComposite() const {
+  return true;
   }
 
 
