@@ -3,7 +3,7 @@
 *                        G I F   C u r s o r   O b j e c t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2005 by Daniel Gehriger.   All Rights Reserved.            *
+* Copyright (C) 2000,2006 by Daniel Gehriger.   All Rights Reserved.            *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGIFCursor.cpp,v 1.29 2005/01/16 16:06:07 fox Exp $                     *
+* $Id: FXGIFCursor.cpp,v 1.33 2006/01/22 17:58:27 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -78,22 +78,22 @@ FXGIFCursor::FXGIFCursor(FXApp* a,const void *pix,FXint hx,FXint hy):FXCursor(a,
 
 
 // Save object to stream
-FXbool FXGIFCursor::savePixels(FXStream& store) const {
+bool FXGIFCursor::savePixels(FXStream& store) const {
   if(fxsaveGIF(store,data,width,height)){
-    return TRUE;
+    return true;
     }
-  return FALSE;
+  return false;
   }
 
 
 // Load object from stream
-FXbool FXGIFCursor::loadPixels(FXStream& store){
+bool FXGIFCursor::loadPixels(FXStream& store){
   if(options&CURSOR_OWNED){FXFREE(&data);}
   if(fxloadGIF(store,data,width,height)){
     options|=CURSOR_OWNED;
-    return TRUE;
+    return true;
     }
-  return FALSE;
+  return false;
   }
 
 }
