@@ -3,7 +3,7 @@
 *                         T a b   B o o k   W i d g e t                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTabBook.cpp,v 1.18 2005/01/16 16:06:07 fox Exp $                       *
+* $Id: FXTabBook.cpp,v 1.25 2006/01/22 17:58:45 fox Exp $                       *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -56,8 +56,8 @@
   - Fix setCurrent() to be like FXSwitcher.
 */
 
-
 #define TABBOOK_MASK       (TABBOOK_SIDEWAYS|TABBOOK_BOTTOMTABS)
+#define REVEAL_PIXELS      20
 
 using namespace FX;
 
@@ -89,7 +89,7 @@ FXTabBook::FXTabBook(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXi
 
 // Get width
 FXint FXTabBook::getDefaultWidth(){
-  register FXint w,wtabs,wmaxtab,wpnls,t,ntabs;
+  register FXint w,wtabs,maxtabw,wpnls,t,ntabs;
   register FXWindow *tab,*pane;
   register FXuint hints;
 
@@ -111,20 +111,20 @@ FXint FXTabBook::getDefaultWidth(){
 
   // Top or bottom tabs
   else{
-    wtabs=wpnls=wmaxtab=ntabs=0;
+    wtabs=wpnls=maxtabw=ntabs=0;
     for(tab=getFirst(); tab && tab->getNext(); tab=tab->getNext()->getNext()){
       pane=tab->getNext();
       if(tab->shown()){
         hints=tab->getLayoutHints();
         if(hints&LAYOUT_FIX_WIDTH) t=tab->getWidth(); else t=tab->getDefaultWidth();
-        if(t>wmaxtab) wmaxtab=t;
+        if(t>maxtabw) maxtabw=t;
         wtabs+=t;
         t=pane->getDefaultWidth();
         if(t>wpnls) wpnls=t;
         ntabs++;
         }
       }
-    if(options&PACK_UNIFORM_WIDTH) wtabs=ntabs*wmaxtab;
+    if(options&PACK_UNIFORM_WIDTH) wtabs=ntabs*maxtabw;
     wtabs+=5;
     w=FXMAX(wtabs,wpnls);
     }
@@ -134,26 +134,26 @@ FXint FXTabBook::getDefaultWidth(){
 
 // Get height
 FXint FXTabBook::getDefaultHeight(){
-  register FXint h,htabs,hmaxtab,hpnls,t,ntabs;
+  register FXint h,htabs,maxtabh,hpnls,t,ntabs;
   register FXWindow *tab,*pane;
   register FXuint hints;
 
   // Left or right tabs
   if(options&TABBOOK_SIDEWAYS){
-    htabs=hpnls=hmaxtab=ntabs=0;
+    htabs=hpnls=maxtabh=ntabs=0;
     for(tab=getFirst(); tab && tab->getNext(); tab=tab->getNext()->getNext()){
       pane=tab->getNext();
       if(tab->shown()){
         hints=tab->getLayoutHints();
         if(hints&LAYOUT_FIX_HEIGHT) t=tab->getHeight(); else t=tab->getDefaultHeight();
-        if(t>hmaxtab) hmaxtab=t;
+        if(t>maxtabh) maxtabh=t;
         htabs+=t;
         t=pane->getDefaultHeight();
         if(t>hpnls) hpnls=t;
         ntabs++;
         }
       }
-    if(options&PACK_UNIFORM_HEIGHT) htabs=ntabs*hmaxtab;
+    if(options&PACK_UNIFORM_HEIGHT) htabs=ntabs*maxtabh;
     htabs+=5;
     h=FXMAX(htabs,hpnls);
     }
@@ -179,7 +179,7 @@ FXint FXTabBook::getDefaultHeight(){
 
 // Recalculate layout
 void FXTabBook::layout(){
-  register int i,x,y,w,h,px,py,pw,ph,wmaxtab,hmaxtab,newcurrent;
+  register FXint i,xx,yy,x,y,w,h,px,py,pw,ph,maxtabw,maxtabh,cumw,cumh,newcurrent;
   register FXWindow *raisepane=NULL;
   register FXWindow *raisetab=NULL;
   register FXWindow *pane,*tab;
@@ -188,16 +188,15 @@ void FXTabBook::layout(){
   newcurrent=-1;
 
   // Measure tabs again
-  wmaxtab=hmaxtab=0;
+  maxtabw=maxtabh=0;
   for(tab=getFirst(),i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
-    pane=tab->getNext();
     if(tab->shown()){
       hints=tab->getLayoutHints();
+      if(newcurrent<0 || i<=current) newcurrent=i;
       if(hints&LAYOUT_FIX_WIDTH) w=tab->getWidth(); else w=tab->getDefaultWidth();
       if(hints&LAYOUT_FIX_HEIGHT) h=tab->getHeight(); else h=tab->getDefaultHeight();
-      if(w>wmaxtab) wmaxtab=w;
-      if(h>hmaxtab) hmaxtab=h;
-      if(newcurrent<0 || i<=current) newcurrent=i;
+      if(w>maxtabw) maxtabw=w;
+      if(h>maxtabh) maxtabh=h;
       }
     }
 
@@ -208,41 +207,88 @@ void FXTabBook::layout(){
   if(options&TABBOOK_SIDEWAYS){
 
     // Place panel
-    px=(options&TABBOOK_BOTTOMTABS) ? border+padleft : border+padleft+wmaxtab-2;
+    px=(options&TABBOOK_BOTTOMTABS) ? border+padleft : border+padleft+maxtabw-2;
     py=border+padtop;
-    pw=width-padleft-padright-(border<<1)-wmaxtab+2;
+    pw=width-padleft-padright-(border<<1)-maxtabw+2;
     ph=height-padtop-padbottom-(border<<1);
 
+    // Scroll as appropriate
+    for(tab=getFirst(),cumh=i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
+      if(tab->shown()){
+        hints=tab->getLayoutHints();
+        if(hints&LAYOUT_FIX_HEIGHT) h=tab->getHeight();
+        else if(options&PACK_UNIFORM_HEIGHT) h=maxtabh;
+        else h=tab->getDefaultHeight();
+        if(i==current){
+          if(tab->getNext()->getNext()){
+            if(cumh+shift+h>ph-REVEAL_PIXELS-2) shift=ph-cumh-h-REVEAL_PIXELS-2;
+            }
+          else{
+            if(cumh+shift+h>ph-2) shift=ph-cumh-h-2;
+            }
+          if(tab->getPrev()){
+            if(cumh+shift<REVEAL_PIXELS+2) shift=REVEAL_PIXELS+2-cumh;
+            }
+          else{
+            if(cumh+shift<2) shift=2-cumh;
+            }
+          }
+        cumh+=h;
+        }
+      }
+
+    // Adjust shift based on space
+    if(shift<ph-cumh-2) shift=ph-cumh-2;
+    if(shift>0) shift=0;
+
     // Place all of the children
-    for(tab=getFirst(),y=py,i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
+    for(tab=getFirst(),yy=py+shift,i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
       pane=tab->getNext();
       if(tab->shown()){
         pane->position(px,py,pw,ph);
         hints=tab->getLayoutHints();
         if(hints&LAYOUT_FIX_WIDTH) w=tab->getWidth();
-        else if(options&PACK_UNIFORM_WIDTH) w=wmaxtab;
+        else if(options&PACK_UNIFORM_WIDTH) w=maxtabw;
         else w=tab->getDefaultWidth();
         if(hints&LAYOUT_FIX_HEIGHT) h=tab->getHeight();
-        else if(options&PACK_UNIFORM_HEIGHT) h=hmaxtab;
+        else if(options&PACK_UNIFORM_HEIGHT) h=maxtabh;
         else h=tab->getDefaultHeight();
-        if(current==i){
+        if(i<current){
+          y=yy+2;
+          if(y+h>py+ph-2) y=py+ph-2-h;
+          if(y<py+2) y=py+2;
+          if(options&TABBOOK_BOTTOMTABS)
+            tab->position(px+pw-4,y+2,w,h);
+          else
+            tab->position(px-w+4,y+2,w,h);
+          tab->raise();
+          pane->hide();
+          yy+=h;
+          }
+        else if(i>current){
+          y=yy+2;
+          if(y+h>py+ph-2) y=py+ph-h-2;
+          if(y<py+2) y=py+2;
+          if(options&TABBOOK_BOTTOMTABS)
+            tab->position(px+pw-4,y+2,w,h);
+          else
+            tab->position(px-w+4,y+2,w,h);
+          tab->lower();
+          pane->hide();
+          yy+=h;
+          }
+        else{
+          y=yy;
+          if(y+h>py+ph-2) y=py+ph-h-2;
+          if(y<py) y=py;
           if(options&TABBOOK_BOTTOMTABS)
             tab->position(px+pw-2,y,w,h);
           else
             tab->position(px-w+2,y,w,h);
           pane->show();
-          raisetab=tab;
           raisepane=pane;
-          y+=h-3;
-          }
-        else{
-          if(options&TABBOOK_BOTTOMTABS)
-            tab->position(px+pw-4,y+2,w,h);
-          else
-            tab->position(px-w+4,y+2,w,h);
-          tab->update(0,0,wmaxtab,h);
-          pane->hide();
-          y+=h;
+          raisetab=tab;
+          yy+=h-3;
           }
         }
       else{
@@ -259,23 +305,79 @@ void FXTabBook::layout(){
 
     // Place panel
     px=border+padleft;
-    py=(options&TABBOOK_BOTTOMTABS) ? border+padtop : border+padtop+hmaxtab-2;
+    py=(options&TABBOOK_BOTTOMTABS) ? border+padtop : border+padtop+maxtabh-2;
     pw=width-padleft-padright-(border<<1);
-    ph=height-padtop-padbottom-(border<<1)-hmaxtab+2;
+    ph=height-padtop-padbottom-(border<<1)-maxtabh+2;
+
+    // Scroll as appropriate
+    for(tab=getFirst(),cumw=i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
+      if(tab->shown()){
+        hints=tab->getLayoutHints();
+        if(hints&LAYOUT_FIX_WIDTH) w=tab->getWidth();
+        else if(options&PACK_UNIFORM_WIDTH) w=maxtabw;
+        else w=tab->getDefaultWidth();
+        if(i==current){
+          if(tab->getNext()->getNext()){
+            if(cumw+shift+w>pw-REVEAL_PIXELS-2) shift=pw-cumw-w-REVEAL_PIXELS-2;
+            }
+          else{
+            if(cumw+shift+w>pw-2) shift=pw-cumw-w-2;
+            }
+          if(tab->getPrev()){
+            if(cumw+shift<REVEAL_PIXELS+2) shift=REVEAL_PIXELS+2-cumw;
+            }
+          else{
+            if(cumw+shift<2) shift=2-cumw;
+            }
+          }
+        cumw+=w;
+        }
+      }
+
+    // Adjust shift based on space
+    if(shift<pw-cumw-2) shift=pw-cumw-2;
+    if(shift>0) shift=0;
 
     // Place all of the children
-    for(tab=getFirst(),x=px,i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
+    for(tab=getFirst(),xx=px+shift,i=0; tab && tab->getNext(); tab=tab->getNext()->getNext(),i++){
       pane=tab->getNext();
       if(tab->shown()){
         pane->position(px,py,pw,ph);
         hints=tab->getLayoutHints();
         if(hints&LAYOUT_FIX_WIDTH) w=tab->getWidth();
-        else if(options&PACK_UNIFORM_WIDTH) w=wmaxtab;
+        else if(options&PACK_UNIFORM_WIDTH) w=maxtabw;
         else w=tab->getDefaultWidth();
         if(hints&LAYOUT_FIX_HEIGHT) h=tab->getHeight();
-        else if(options&PACK_UNIFORM_HEIGHT) h=hmaxtab;
+        else if(options&PACK_UNIFORM_HEIGHT) h=maxtabh;
         else h=tab->getDefaultHeight();
-        if(current==i){
+        if(i<current){
+          x=xx+2;
+          if(x+w>px+pw-2) x=px+pw-2-w;
+          if(x<px+2) x=px+2;
+          if(options&TABBOOK_BOTTOMTABS)
+            tab->position(x,py+ph-4,w,h);
+          else
+            tab->position(x,py-h+4,w,h);
+          tab->raise();
+          pane->hide();
+          xx+=w;
+          }
+        else if(i>current){
+          x=xx+2;
+          if(x+w>px+pw-2) x=px+pw-w-2;
+          if(x<px+2) x=px+2;
+          if(options&TABBOOK_BOTTOMTABS)
+            tab->position(x,py+ph-4,w,h);
+          else
+            tab->position(x,py-h+4,w,h);
+          tab->lower();
+          pane->hide();
+          xx+=w;
+          }
+        else{
+          x=xx;
+          if(x+w>px+pw-2) x=px+pw-w-2;
+          if(x<px) x=px;
           if(options&TABBOOK_BOTTOMTABS)
             tab->position(x,py+ph-2,w,h);
           else
@@ -283,15 +385,7 @@ void FXTabBook::layout(){
           pane->show();
           raisepane=pane;
           raisetab=tab;
-          x+=w-3;
-          }
-        else{
-          if(options&TABBOOK_BOTTOMTABS)
-            tab->position(x+2,py+ph-4,w,h);
-          else
-            tab->position(x+2,py-h+4,w,h);
-          pane->hide();
-          x+=w;
+          xx+=w-3;
           }
         }
       else{
