@@ -3,7 +3,7 @@
 *                         P N G    I n p u t / O u t p u t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxpngio.cpp,v 1.35.2.1 2005/11/19 20:22:25 fox Exp $                         *
+* $Id: fxpngio.cpp,v 1.40 2006/01/22 17:58:54 fox Exp $                         *
 ********************************************************************************/
 #include "fxver.h"
 #include "fxdefs.h"
@@ -45,9 +45,9 @@ using namespace FX;
 namespace FX {
 
 
-extern FXAPI FXbool fxcheckPNG(FXStream& store);
-extern FXAPI FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height);
-extern FXAPI FXbool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height);
+extern FXAPI bool fxcheckPNG(FXStream& store);
+extern FXAPI bool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height);
+extern FXAPI bool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height);
 
 
 #ifdef HAVE_PNG_H
@@ -72,23 +72,20 @@ static void user_flush_fn(png_structp ){ }
 
 
 // Custom error handler; this is unrecoverable
-static void user_error_fn(png_structp png_ptr,png_const_charp message){
+static void user_error_fn(png_structp png_ptr,png_const_charp){
   FXStream* store=(FXStream*)png_get_error_ptr(png_ptr);
   store->setError(FXStreamFormat);                      // Flag this as a format error in FXStream
-  FXTRACE((100,"Error in png: %s\n",message));
   longjmp(png_ptr->jmpbuf,1);                           // Bail out
   }
 
 
 // Custom warning handler; we assume this is recoverable
-static void user_warning_fn(png_structp,png_const_charp message){
-  //FXStream* store=(FXStream*)png_get_error_ptr(png_ptr);
-  FXTRACE((100,"Warning in png: %s\n",message));
+static void user_warning_fn(png_structp,png_const_charp){
   }
 
 
 // Check if stream contains a PNG
-FXbool fxcheckPNG(FXStream& store){
+bool fxcheckPNG(FXStream& store){
   FXuchar signature[8];
   store.load(signature,8);
   store.position(-8,FXFromCurrent);
@@ -97,11 +94,11 @@ FXbool fxcheckPNG(FXStream& store){
 
 
 // Load a PNG image
-FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
+bool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
   png_structp png_ptr;
   png_infop info_ptr;
   png_uint_32 ww,hh,i;
-  int bit_depth,color_type,interlace_type,number_passes;
+  int bit_depth,color_type,interlace_type;
   png_bytep *row_pointers;
 
   // Null out
@@ -111,13 +108,13 @@ FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
 
   // Create png_struct
   png_ptr=png_create_read_struct(PNG_LIBPNG_VER_STRING,&store,user_error_fn,user_warning_fn);
-  if(!png_ptr) return FALSE;
+  if(!png_ptr) return false;
 
   // Allocate/initialize the memory for image information
   info_ptr=png_create_info_struct(png_ptr);
   if(!info_ptr){
     png_destroy_read_struct(&png_ptr,(png_infopp)NULL,(png_infopp)NULL);
-    return FALSE;
+    return false;
     }
 
   // Set error handling
@@ -125,7 +122,7 @@ FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
 
     // Free all of the memory associated with the png_ptr and info_ptr
     png_destroy_read_struct(&png_ptr,&info_ptr,(png_infopp)NULL);
-    return FALSE;
+    return false;
     }
 
   // Using replacement read functions
@@ -162,7 +159,7 @@ FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
   png_set_filler(png_ptr,0xff,PNG_FILLER_AFTER);
 
   // Turn on interlace handling
-  number_passes=png_set_interlace_handling(png_ptr);
+  png_set_interlace_handling(png_ptr);
 
   // Update image info based on transformations
   png_read_update_info(png_ptr,info_ptr);
@@ -171,7 +168,7 @@ FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
   FXMALLOC(&data,FXColor,hh*ww);
   if(!data){
     png_destroy_read_struct(&png_ptr,&info_ptr,(png_infopp)NULL);
-    return FALSE;
+    return false;
     }
 
   // Row pointers
@@ -179,7 +176,7 @@ FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
   if(!row_pointers){
     FXFREE(&data);
     png_destroy_read_struct(&png_ptr,&info_ptr,(png_infopp)NULL);
-    return FALSE;
+    return false;
     }
 
   // Set up row pointers
@@ -204,7 +201,7 @@ FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
   width=ww;
   height=hh;
 
-  return TRUE;
+  return true;
   }
 
 
@@ -213,30 +210,30 @@ FXbool fxloadPNG(FXStream& store,FXColor*& data,FXint& width,FXint& height){
 
 
 // Save a PNG image
-FXbool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height){
+bool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height){
   png_structp png_ptr;
   png_infop info_ptr;
   png_bytep *row_pointers;
   int i;
 
   // Must make sense
-  if(!data || width<=0 || height<=0) return FALSE;
+  if(!data || width<=0 || height<=0) return false;
 
   // Create and initialize the png_struct with the desired error handler functions.
   png_ptr=png_create_write_struct(PNG_LIBPNG_VER_STRING,&store,user_error_fn,user_warning_fn);
-  if(!png_ptr) return FALSE;
+  if(!png_ptr) return false;
 
   // Allocate/initialize the image information data.
   info_ptr=png_create_info_struct(png_ptr);
   if(!info_ptr){
     png_destroy_write_struct(&png_ptr,(png_infopp)NULL);
-    return FALSE;
+    return false;
     }
 
   // Set error handling.
   if(setjmp(png_ptr->jmpbuf)){
     png_destroy_write_struct(&png_ptr,&info_ptr);
-    return FALSE;
+    return false;
     }
 
   // Using replacement read functions
@@ -250,7 +247,7 @@ FXbool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height){
   FXMALLOC(&row_pointers,png_bytep,height);
   if(!row_pointers){
     png_destroy_write_struct(&png_ptr,&info_ptr);
-    return FALSE;
+    return false;
     }
 
   // Set up row pointers
@@ -270,7 +267,7 @@ FXbool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height){
   // Get rid of it
   FXFREE(&row_pointers);
 
-  return TRUE;
+  return true;
   }
 
 
@@ -281,13 +278,13 @@ FXbool fxsavePNG(FXStream& store,const FXColor* data,FXint width,FXint height){
 
 
 // Check if stream contains a PNG
-FXbool fxcheckPNG(FXStream&){
-  return FALSE;
+bool fxcheckPNG(FXStream&){
+  return false;
   }
-  
-  
+
+
 // Stub routine
-FXbool fxloadPNG(FXStream&,FXColor*& data,FXint& width,FXint& height){
+bool fxloadPNG(FXStream&,FXColor*& data,FXint& width,FXint& height){
   static const FXuchar png_bits[] = {
    0xff, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x80, 0xfd, 0xff, 0xff, 0xbf,
    0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
@@ -307,13 +304,13 @@ FXbool fxloadPNG(FXStream&,FXColor*& data,FXint& width,FXint& height){
     }
   width=32;
   height=32;
-  return TRUE;
+  return true;
   }
 
 
 // Stub routine
-FXbool fxsavePNG(FXStream&,const FXColor*,FXint,FXint){
-  return FALSE;
+bool fxsavePNG(FXStream&,const FXColor*,FXint,FXint){
+  return false;
   }
 
 
